@@ -42,7 +42,9 @@
 
 #if defined(POLARSSL_HAVE_ASM)
 
-#if defined(__GNUC__)
+/* armcc5 --gnu defines __GNUC__ but doesn't support GNU's extended asm */
+#if defined(__GNUC__) && \
+    ( !defined(__ARMCC_VERSION) || __ARMCC_VERSION >= 6000000 )
 #if defined(__i386__)
 
 #define MULADDC_INIT                        \
